/*
 * $Id: apitest.prg 9279 2011-02-14 18:06:32Z druzus $
 */
/*
   Program for:
    - testing availability of functions in a given version of ace32.dll
    - testing availability of interface functions in ace32.c
    - function lists are always based on the latest version of ace32.dll
      as documented in the corresponring ace.h. This test program uses
      version 10.0.0.3
    - the purpose of this program is to give info to programmers to not use
      functions with WARNING attribute as a consequence of the ace32.dll she
      is using.
    - WARNING is emmited when either function is not found in ace32.dll OR
      there is no documentation (prototype) in ace.h
    - compile and direct output to a text file ie.: apitest >api.txt
   By: Andi Jahja
*/

PROCEDURE MAIN()

   LOCAL aFunc:= ACE32DLL_FuncList()
   LOCAL i, j := len( aFunc ), lInit := .F.
   LOCAL aAce32_C := ACE32LIB_FuncList(), w := Len( aAce32_C )

   OutStd( "=============================================" +  hb_osnewline() )
   OutStd( "Availability of interface function in ace32.c" + hb_osnewline() )
   OutStd( "=============================================" +  hb_osnewline() )
   OutStd( hb_osnewline() )

   FOR i := 1 to j
      IF aScan( aAce32_C, { |e| e == aFunc[i] } ) > 0
         OutStd( "Interface Function: " + aFunc[i] + " OK.." + hb_osnewline() )
      ELSE
         OutStd( ">>> WARNING: Interface Function: " + aFunc[i] + " NOT FOUND..<<<" + hb_osnewline() )
      ENDIF
   NEXT

   OutStd( hb_osnewline() )

   OutStd( "==============================" +  hb_osnewline() )
   OutStd( "Using ace32.dll version " + ADSVERSION() + hb_osnewline() )
   OutStd( "==============================" +  hb_osnewline() + hb_osnewline() )

   FOR i := 1 TO j
      IF TESTACEAPI( aFunc[i] )
         OutStd( aFunc[i] + " OK .." + hb_osnewline() )
      ELSE
         OutStd( ">>>  WARNING: " + aFunc[i] + " NOT FOUND .. <<<" + hb_osnewline() )
      ENDIF
   NEXT


   RETURN

STATIC FUNCTION ACE32DLL_FUNCLIST()

   //Based on ACE32.LIB Version 10

   LOCAL aFuncList := {}

   AAdd( aFuncList, "AdsAccessVfpSystemField" )
   AAdd( aFuncList, "AdsActivateAOF" )
   AAdd( aFuncList, "AdsAddCustomKey" )
   AAdd( aFuncList, "AdsAddToAOF" )
   AAdd( aFuncList, "AdsAppendRecord" )
   AAdd( aFuncList, "AdsApplicationExit" )
   AAdd( aFuncList, "AdsAreTriggersEnabled" )
   AAdd( aFuncList, "AdsAtBOF" )
   AAdd( aFuncList, "AdsAtEOF" )
   AAdd( aFuncList, "AdsBeginTransaction" )
   AAdd( aFuncList, "AdsBinaryToFile" )
   AAdd( aFuncList, "AdsBuildKeyFromRecord" )
   AAdd( aFuncList, "AdsBuildRawKey" )
   AAdd( aFuncList, "AdsBuildRawKey100" )
   AAdd( aFuncList, "AdsCacheOpenCursors" )
   AAdd( aFuncList, "AdsCacheOpenTables" )
   AAdd( aFuncList, "AdsCacheRecords" )
   AAdd( aFuncList, "AdsCancelUpdate" )
   AAdd( aFuncList, "AdsCancelUpdate90" )
   AAdd( aFuncList, "AdsCheckExistence" )
   AAdd( aFuncList, "AdsClearAOF" )
   AAdd( aFuncList, "AdsClearAllScopes" )
   AAdd( aFuncList, "AdsClearCallbackFunction" )
   AAdd( aFuncList, "AdsClearCursorAOF" )
   AAdd( aFuncList, "AdsClearDefault" )
   AAdd( aFuncList, "AdsClearFilter" )
   AAdd( aFuncList, "AdsClearLastError" )
   AAdd( aFuncList, "AdsClearProgressCallback" )
   AAdd( aFuncList, "AdsClearRecordBuffer" )
   AAdd( aFuncList, "AdsClearRelation" )
   AAdd( aFuncList, "AdsClearSQLAbortFunc" )
   AAdd( aFuncList, "AdsClearSQLParams" )
   AAdd( aFuncList, "AdsClearScope" )
   AAdd( aFuncList, "AdsCloneTable" )
   AAdd( aFuncList, "AdsCloseAllIndexes" )
   AAdd( aFuncList, "AdsCloseAllTables" )
   AAdd( aFuncList, "AdsCloseCachedTables" )
   AAdd( aFuncList, "AdsCloseCachedTrigStatements" )
   AAdd( aFuncList, "AdsCloseFile" )
   AAdd( aFuncList, "AdsCloseIndex" )
   AAdd( aFuncList, "AdsCloseSQLStatement" )
   AAdd( aFuncList, "AdsCloseTable" )
   AAdd( aFuncList, "AdsCommitTransaction" )
   AAdd( aFuncList, "AdsCompareBookmarks" )
   AAdd( aFuncList, "AdsConnect" )
   AAdd( aFuncList, "AdsConnect26" )
   AAdd( aFuncList, "AdsConnect60" )
   AAdd( aFuncList, "AdsContinue" )
   AAdd( aFuncList, "AdsConvertCodePageToUnicode" )
   AAdd( aFuncList, "AdsConvertDateToJulian" )
   AAdd( aFuncList, "AdsConvertJulianToString" )
   AAdd( aFuncList, "AdsConvertKeyToDouble" )
   AAdd( aFuncList, "AdsConvertMillisecondsToString" )
   AAdd( aFuncList, "AdsConvertStringToJulian" )
   AAdd( aFuncList, "AdsConvertStringToMilliseconds" )
   AAdd( aFuncList, "AdsConvertTable" )
   AAdd( aFuncList, "AdsConvertUnicodeToCodePage" )
   AAdd( aFuncList, "AdsCopyTable" )
   AAdd( aFuncList, "AdsCopyTableContents" )
   AAdd( aFuncList, "AdsCopyTableStructure" )
   AAdd( aFuncList, "AdsCopyTableStructure81" )
   AAdd( aFuncList, "AdsCopyTableTop" )
   AAdd( aFuncList, "AdsCopyTableTop100" )
   AAdd( aFuncList, "AdsCreateCriticalSection" )
   AAdd( aFuncList, "AdsCreateFTSIndex" )
   AAdd( aFuncList, "AdsCreateIndex" )
   AAdd( aFuncList, "AdsCreateIndex61" )
   AAdd( aFuncList, "AdsCreateIndex90" )
   AAdd( aFuncList, "AdsCreateMemTable" )
   AAdd( aFuncList, "AdsCreateMemTable90" )
   AAdd( aFuncList, "AdsCreateSQLStatement" )
   AAdd( aFuncList, "AdsCreateSavepoint" )
   AAdd( aFuncList, "AdsCreateTable" )
   AAdd( aFuncList, "AdsCreateTable71" )
   AAdd( aFuncList, "AdsCreateTable90" )
   AAdd( aFuncList, "AdsCustomizeAOF" )
   AAdd( aFuncList, "AdsDBFDateToString" )
   AAdd( aFuncList, "AdsDDAddIndexFile" )
   AAdd( aFuncList, "AdsDDAddProcedure" )
   AAdd( aFuncList, "AdsDDAddProcedure100" )
   AAdd( aFuncList, "AdsDDAddTable" )
   AAdd( aFuncList, "AdsDDAddTable90" )
   AAdd( aFuncList, "AdsDDAddUserToGroup" )
   AAdd( aFuncList, "AdsDDAddView" )
   AAdd( aFuncList, "AdsDDAddView100" )
   AAdd( aFuncList, "AdsDDAutoCreateIndex" )
   AAdd( aFuncList, "AdsDDAutoCreateTable" )
   AAdd( aFuncList, "AdsDDClose" )
   AAdd( aFuncList, "AdsDDCreate" )
   AAdd( aFuncList, "AdsDDCreateASA" )
   AAdd( aFuncList, "AdsDDCreateArticle" )
   AAdd( aFuncList, "AdsDDCreateArticle100" )
   AAdd( aFuncList, "AdsDDCreateFunction" )
   AAdd( aFuncList, "AdsDDCreateFunction100" )
   AAdd( aFuncList, "AdsDDCreateLink" )
   AAdd( aFuncList, "AdsDDCreatePackage" )
   AAdd( aFuncList, "AdsDDCreatePublication" )
   AAdd( aFuncList, "AdsDDCreateRefIntegrity" )
   AAdd( aFuncList, "AdsDDCreateRefIntegrity62" )
   AAdd( aFuncList, "AdsDDCreateSubscription" )
   AAdd( aFuncList, "AdsDDCreateTrigger" )
   AAdd( aFuncList, "AdsDDCreateTrigger100" )
   AAdd( aFuncList, "AdsDDCreateUser" )
   AAdd( aFuncList, "AdsDDCreateUserGroup" )
   AAdd( aFuncList, "AdsDDDeleteArticle" )
   AAdd( aFuncList, "AdsDDDeleteIndex" )
   AAdd( aFuncList, "AdsDDDeletePublication" )
   AAdd( aFuncList, "AdsDDDeleteSubscription" )
   AAdd( aFuncList, "AdsDDDeleteUser" )
   AAdd( aFuncList, "AdsDDDeleteUserGroup" )
   AAdd( aFuncList, "AdsDDDeployDatabase" )
   AAdd( aFuncList, "AdsDDDisableTriggers" )
   AAdd( aFuncList, "AdsDDDropFunction" )
   AAdd( aFuncList, "AdsDDDropLink" )
   AAdd( aFuncList, "AdsDDDropPackage" )
   AAdd( aFuncList, "AdsDDEnableTriggers" )
   AAdd( aFuncList, "AdsDDExecuteProcedure" )
   AAdd( aFuncList, "AdsDDFindClose" )
   AAdd( aFuncList, "AdsDDFindFirstObject" )
   AAdd( aFuncList, "AdsDDFindNextObject" )
   AAdd( aFuncList, "AdsDDFreeTable" )
   AAdd( aFuncList, "AdsDDGetArticleProperty" )
   AAdd( aFuncList, "AdsDDGetDatabaseProperty" )
   AAdd( aFuncList, "AdsDDGetFieldProperty" )
   AAdd( aFuncList, "AdsDDGetIndexFileProperty" )
   AAdd( aFuncList, "AdsDDGetIndexProperty" )
   AAdd( aFuncList, "AdsDDGetLinkProperty" )
   AAdd( aFuncList, "AdsDDGetObjectProperty" )
   AAdd( aFuncList, "AdsDDGetObjectProperty100" )
   AAdd( aFuncList, "AdsDDGetPermissions" )
   AAdd( aFuncList, "AdsDDGetProcInterfaceVersion" )
   AAdd( aFuncList, "AdsDDGetProcedureProperty" )
   AAdd( aFuncList, "AdsDDGetPublicationProperty" )
   AAdd( aFuncList, "AdsDDGetRefIntegrityProperty" )
   AAdd( aFuncList, "AdsDDGetSubscriptionProperty" )
   AAdd( aFuncList, "AdsDDGetTableOpenOptions" )
   AAdd( aFuncList, "AdsDDGetTableProperty" )
   AAdd( aFuncList, "AdsDDGetTriggerProperty" )
   AAdd( aFuncList, "AdsDDGetUserGroupProperty" )
   AAdd( aFuncList, "AdsDDGetUserProperty" )
   AAdd( aFuncList, "AdsDDGetViewProperty" )
   AAdd( aFuncList, "AdsDDGrantPermission" )
   AAdd( aFuncList, "AdsDDModifyLink" )
   AAdd( aFuncList, "AdsDDMoveObjectFile" )
   AAdd( aFuncList, "AdsDDOpen" )
   AAdd( aFuncList, "AdsDDRemoveIndexFile" )
   AAdd( aFuncList, "AdsDDRemoveProcedure" )
   AAdd( aFuncList, "AdsDDRemoveRefIntegrity" )
   AAdd( aFuncList, "AdsDDRemoveTable" )
   AAdd( aFuncList, "AdsDDRemoveTrigger" )
   AAdd( aFuncList, "AdsDDRemoveUserFromGroup" )
   AAdd( aFuncList, "AdsDDRemoveView" )
   AAdd( aFuncList, "AdsDDRenameObject" )
   AAdd( aFuncList, "AdsDDRevokePermission" )
   AAdd( aFuncList, "AdsDDSetActiveDictionary" )
   AAdd( aFuncList, "AdsDDSetArticleProperty" )
   AAdd( aFuncList, "AdsDDSetDatabaseProperty" )
   AAdd( aFuncList, "AdsDDSetFieldProperty" )
   AAdd( aFuncList, "AdsDDSetIndexProperty" )
   AAdd( aFuncList, "AdsDDSetObjectAccessRights" )
   AAdd( aFuncList, "AdsDDSetObjectProperty" )
   AAdd( aFuncList, "AdsDDSetObjectProperty100" )
   AAdd( aFuncList, "AdsDDSetProcedureProperty" )
   AAdd( aFuncList, "AdsDDSetPublicationProperty" )
   AAdd( aFuncList, "AdsDDSetSubscriptionProperty" )
   AAdd( aFuncList, "AdsDDSetTableProperty" )
   AAdd( aFuncList, "AdsDDSetTriggerProperty" )
   AAdd( aFuncList, "AdsDDSetUserGroupProperty" )
   AAdd( aFuncList, "AdsDDSetUserProperty" )
   AAdd( aFuncList, "AdsDDSetViewProperty" )
   AAdd( aFuncList, "AdsDDVerifyUserRights" )
   AAdd( aFuncList, "AdsDeactivateAOF" )
   AAdd( aFuncList, "AdsDecryptRecord" )
   AAdd( aFuncList, "AdsDecryptTable" )
   AAdd( aFuncList, "AdsDeleteCustomKey" )
   AAdd( aFuncList, "AdsDeleteFile" )
   AAdd( aFuncList, "AdsDeleteIndex" )
   AAdd( aFuncList, "AdsDeleteRecord" )
   AAdd( aFuncList, "AdsDeleteTable" )
   AAdd( aFuncList, "AdsDisableAutoIncEnforcement" )
   AAdd( aFuncList, "AdsDisableEncryption" )
   AAdd( aFuncList, "AdsDisableLocalConnections" )
   AAdd( aFuncList, "AdsDisableRI" )
   AAdd( aFuncList, "AdsDisableUniqueEnforcement" )
   AAdd( aFuncList, "AdsDisconnect" )
   AAdd( aFuncList, "AdsEcho" )
   AAdd( aFuncList, "AdsEnableAutoIncEnforcement" )
   AAdd( aFuncList, "AdsEnableEncryption" )
   AAdd( aFuncList, "AdsEnableRI" )
   AAdd( aFuncList, "AdsEnableUniqueEnforcement" )
   AAdd( aFuncList, "AdsEncryptRecord" )
   AAdd( aFuncList, "AdsEncryptTable" )
   AAdd( aFuncList, "AdsEvalAOF" )
   AAdd( aFuncList, "AdsEvalAOF100" )
   AAdd( aFuncList, "AdsEvalExpr" )
   AAdd( aFuncList, "AdsEvalLogicalExpr" )
   AAdd( aFuncList, "AdsEvalLogicalExprW" )
   AAdd( aFuncList, "AdsEvalNumericExpr" )
   AAdd( aFuncList, "AdsEvalStringExpr" )
   AAdd( aFuncList, "AdsEvalTestExpr" )
   AAdd( aFuncList, "AdsExecuteSQL" )
   AAdd( aFuncList, "AdsExecuteSQLDirect" )
   AAdd( aFuncList, "AdsExecuteSQLDirectW" )
   AAdd( aFuncList, "AdsExpressionLongToShort" )
   AAdd( aFuncList, "AdsExpressionLongToShort100" )
   AAdd( aFuncList, "AdsExpressionLongToShort90" )
   AAdd( aFuncList, "AdsExpressionShortToLong" )
   AAdd( aFuncList, "AdsExpressionShortToLong90" )
   AAdd( aFuncList, "AdsExtractKey" )
   AAdd( aFuncList, "AdsExtractPathPart" )
   AAdd( aFuncList, "AdsFailedTransactionRecovery" )
   AAdd( aFuncList, "AdsFileToBinary" )
   AAdd( aFuncList, "AdsFindClose" )
   AAdd( aFuncList, "AdsFindConnection" )
   AAdd( aFuncList, "AdsFindConnection25" )
   AAdd( aFuncList, "AdsFindFirstTable" )
   AAdd( aFuncList, "AdsFindFirstTable62" )
   AAdd( aFuncList, "AdsFindNextTable" )
   AAdd( aFuncList, "AdsFindNextTable62" )
   AAdd( aFuncList, "AdsFindServers" )
   AAdd( aFuncList, "AdsFlushFileBuffers" )
   AAdd( aFuncList, "AdsFreeExpr" )
   AAdd( aFuncList, "AdsFreeTokenBuffer" )
   AAdd( aFuncList, "AdsGetAOF" )
   AAdd( aFuncList, "AdsGetAOF100" )
   AAdd( aFuncList, "AdsGetAOFOptLevel" )
   AAdd( aFuncList, "AdsGetAOFOptLevel100" )
   AAdd( aFuncList, "AdsGetActiveLinkInfo" )
   AAdd( aFuncList, "AdsGetAllIndexes" )
   AAdd( aFuncList, "AdsGetAllLocks" )
   AAdd( aFuncList, "AdsGetAllTables" )
   AAdd( aFuncList, "AdsGetBaseFieldName" )
   AAdd( aFuncList, "AdsGetBaseFieldNum" )
   AAdd( aFuncList, "AdsGetBinary" )
   AAdd( aFuncList, "AdsGetBinaryLength" )
   AAdd( aFuncList, "AdsGetBookmark" )
   AAdd( aFuncList, "AdsGetBookmark60" )
   AAdd( aFuncList, "AdsGetBookmarkLength" )
   AAdd( aFuncList, "AdsGetCollation" )
   AAdd( aFuncList, "AdsGetCollationLang" )
   AAdd( aFuncList, "AdsGetColumnPermissions" )
   AAdd( aFuncList, "AdsGetConnectionPath" )
   AAdd( aFuncList, "AdsGetConnectionProperty" )
   AAdd( aFuncList, "AdsGetConnectionType" )
   AAdd( aFuncList, "AdsGetCursorAOF" )
   AAdd( aFuncList, "AdsGetDataLength" )
   AAdd( aFuncList, "AdsGetDate" )
   AAdd( aFuncList, "AdsGetDateFormat" )
   AAdd( aFuncList, "AdsGetDateFormat60" )
   AAdd( aFuncList, "AdsGetDecimals" )
   AAdd( aFuncList, "AdsGetDefault" )
   AAdd( aFuncList, "AdsGetDeleted" )
   AAdd( aFuncList, "AdsGetDouble" )
   AAdd( aFuncList, "AdsGetEpoch" )
   AAdd( aFuncList, "AdsGetErrorString" )
   AAdd( aFuncList, "AdsGetExact" )
   AAdd( aFuncList, "AdsGetExact22" )
   AAdd( aFuncList, "AdsGetFTSIndexInfo" )
   AAdd( aFuncList, "AdsGetFTSIndexes" )
   AAdd( aFuncList, "AdsGetFTSScore" )
   AAdd( aFuncList, "AdsGetField" )
   AAdd( aFuncList, "AdsGetFieldDecimals" )
   AAdd( aFuncList, "AdsGetFieldLength" )
   AAdd( aFuncList, "AdsGetFieldLength100" )
   AAdd( aFuncList, "AdsGetFieldName" )
   AAdd( aFuncList, "AdsGetFieldNum" )
   AAdd( aFuncList, "AdsGetFieldOffset" )
   AAdd( aFuncList, "AdsGetFieldRaw" )
   AAdd( aFuncList, "AdsGetFieldType" )
   AAdd( aFuncList, "AdsGetFieldW" )
   AAdd( aFuncList, "AdsGetFilePosition" )
   AAdd( aFuncList, "AdsGetFilter" )
   AAdd( aFuncList, "AdsGetHandleLong" )
   AAdd( aFuncList, "AdsGetHandleType" )
   AAdd( aFuncList, "AdsGetIndexCollation" )
   AAdd( aFuncList, "AdsGetIndexCondition" )
   AAdd( aFuncList, "AdsGetIndexExpr" )
   AAdd( aFuncList, "AdsGetIndexFilename" )
   AAdd( aFuncList, "AdsGetIndexFlags" )
   AAdd( aFuncList, "AdsGetIndexHandle" )
   AAdd( aFuncList, "AdsGetIndexHandleByExpr" )
   AAdd( aFuncList, "AdsGetIndexHandleByOrder" )
   AAdd( aFuncList, "AdsGetIndexName" )
   AAdd( aFuncList, "AdsGetIndexOrderByHandle" )
   AAdd( aFuncList, "AdsGetIndexPageSize" )
   AAdd( aFuncList, "AdsGetIntProperty" )
   AAdd( aFuncList, "AdsGetJulian" )
   AAdd( aFuncList, "AdsGetKeyColumn" )
   AAdd( aFuncList, "AdsGetKeyCount" )
   AAdd( aFuncList, "AdsGetKeyLength" )
   AAdd( aFuncList, "AdsGetKeyNum" )
   AAdd( aFuncList, "AdsGetKeyType" )
   AAdd( aFuncList, "AdsGetLastAutoinc" )
   AAdd( aFuncList, "AdsGetLastError" )
   AAdd( aFuncList, "AdsGetLastTableUpdate" )
   AAdd( aFuncList, "AdsGetLibraryVersion" )
   AAdd( aFuncList, "AdsGetLogical" )
   AAdd( aFuncList, "AdsGetLong" )
   AAdd( aFuncList, "AdsGetLongLong" )
   AAdd( aFuncList, "AdsGetMemoBlockSize" )
   AAdd( aFuncList, "AdsGetMemoDataType" )
   AAdd( aFuncList, "AdsGetMemoLength" )
   AAdd( aFuncList, "AdsGetMilliseconds" )
   AAdd( aFuncList, "AdsGetMoney" )
   AAdd( aFuncList, "AdsGetNullRecord" )
   AAdd( aFuncList, "AdsGetNumActiveLinks" )
   AAdd( aFuncList, "AdsGetNumFTSIndexes" )
   AAdd( aFuncList, "AdsGetNumFields" )
   AAdd( aFuncList, "AdsGetNumIndexes" )
   AAdd( aFuncList, "AdsGetNumLocks" )
   AAdd( aFuncList, "AdsGetNumOpenTables" )
   AAdd( aFuncList, "AdsGetNumParams" )
   AAdd( aFuncList, "AdsGetNumSegments" )
   AAdd( aFuncList, "AdsGetPreparedFields" )
   AAdd( aFuncList, "AdsGetRILookupInfo" )
   AAdd( aFuncList, "AdsGetROWIDPrefix" )
   AAdd( aFuncList, "AdsGetRecord" )
   AAdd( aFuncList, "AdsGetRecordCRC" )
   AAdd( aFuncList, "AdsGetRecordCount" )
   AAdd( aFuncList, "AdsGetRecordLength" )
   AAdd( aFuncList, "AdsGetRecordNum" )
   AAdd( aFuncList, "AdsGetRecordPointer" )
   AAdd( aFuncList, "AdsGetRelKeyPos" )
   AAdd( aFuncList, "AdsGetSQLStatement" )
   AAdd( aFuncList, "AdsGetSQLStatementHandle" )
   AAdd( aFuncList, "AdsGetSQLStmtParams" )
   AAdd( aFuncList, "AdsGetScope" )
   AAdd( aFuncList, "AdsGetSearchPath" )
   AAdd( aFuncList, "AdsGetSegmentFieldNumbers" )
   AAdd( aFuncList, "AdsGetSegmentFieldname" )
   AAdd( aFuncList, "AdsGetSegmentOffset" )
   AAdd( aFuncList, "AdsGetServerName" )
   AAdd( aFuncList, "AdsGetServerTime" )
   AAdd( aFuncList, "AdsGetShort" )
   AAdd( aFuncList, "AdsGetString" )
   AAdd( aFuncList, "AdsGetStringW" )
   AAdd( aFuncList, "AdsGetTableAlias" )
   AAdd( aFuncList, "AdsGetTableCharType" )
   AAdd( aFuncList, "AdsGetTableCollation" )
   AAdd( aFuncList, "AdsGetTableConnection" )
   AAdd( aFuncList, "AdsGetTableCreationString" )
   AAdd( aFuncList, "AdsGetTableFilename" )
   AAdd( aFuncList, "AdsGetTableHandle" )
   AAdd( aFuncList, "AdsGetTableHandle25" )
   AAdd( aFuncList, "AdsGetTableLockType" )
   AAdd( aFuncList, "AdsGetTableMemoSize" )
   AAdd( aFuncList, "AdsGetTableOpenOptions" )
   AAdd( aFuncList, "AdsGetTableRights" )
   AAdd( aFuncList, "AdsGetTableType" )
   AAdd( aFuncList, "AdsGetTableWAN" )
   AAdd( aFuncList, "AdsGetTime" )
   AAdd( aFuncList, "AdsGetTransactionCount" )
   AAdd( aFuncList, "AdsGetVersion" )
   AAdd( aFuncList, "AdsGotoBOF" )
   AAdd( aFuncList, "AdsGotoBookmark" )
   AAdd( aFuncList, "AdsGotoBookmark60" )
   AAdd( aFuncList, "AdsGotoBottom" )
   AAdd( aFuncList, "AdsGotoEOF" )
   AAdd( aFuncList, "AdsGotoRecord" )
   AAdd( aFuncList, "AdsGotoTop" )
   AAdd( aFuncList, "AdsImageToClipboard" )
   AAdd( aFuncList, "AdsImageToHBitmap" )
   AAdd( aFuncList, "AdsInTransaction" )
   AAdd( aFuncList, "AdsInitRawKey" )
   AAdd( aFuncList, "AdsInitTokenBuffer" )
   AAdd( aFuncList, "AdsIsConnectionAlive" )
   AAdd( aFuncList, "AdsIsEmpty" )
   AAdd( aFuncList, "AdsIsEncryptionEnabled" )
   AAdd( aFuncList, "AdsIsExprValid" )
   AAdd( aFuncList, "AdsIsFieldBinary" )
   AAdd( aFuncList, "AdsIsFound" )
   AAdd( aFuncList, "AdsIsIndexCandidate" )
   AAdd( aFuncList, "AdsIsIndexCompound" )
   AAdd( aFuncList, "AdsIsIndexCustom" )
   AAdd( aFuncList, "AdsIsIndexDescending" )
   AAdd( aFuncList, "AdsIsIndexExprValid" )
   AAdd( aFuncList, "AdsIsIndexFTS" )
   AAdd( aFuncList, "AdsIsIndexNullable" )
   AAdd( aFuncList, "AdsIsIndexPrimaryKey" )
   AAdd( aFuncList, "AdsIsIndexUnique" )
   AAdd( aFuncList, "AdsIsIndexUserDefined" )
   AAdd( aFuncList, "AdsIsNull" )
   AAdd( aFuncList, "AdsIsNullable" )
   AAdd( aFuncList, "AdsIsRecordDeleted" )
   AAdd( aFuncList, "AdsIsRecordEncrypted" )
   AAdd( aFuncList, "AdsIsRecordInAOF" )
   AAdd( aFuncList, "AdsIsRecordLocked" )
   AAdd( aFuncList, "AdsIsRecordVisible" )
   AAdd( aFuncList, "AdsIsSegmentDescending" )
   AAdd( aFuncList, "AdsIsServerLoaded" )
   AAdd( aFuncList, "AdsIsTableEncrypted" )
   AAdd( aFuncList, "AdsIsTableLocked" )
   AAdd( aFuncList, "AdsIsTableTransactionFree" )
   AAdd( aFuncList, "AdsLocate" )
   AAdd( aFuncList, "AdsLockRecord" )
   AAdd( aFuncList, "AdsLockRecordImplicitly" )
   AAdd( aFuncList, "AdsLockTable" )
   AAdd( aFuncList, "AdsLookupKey" )
   AAdd( aFuncList, "AdsMakeNameUnique" )
   AAdd( aFuncList, "AdsMemCompare" )
   AAdd( aFuncList, "AdsMemCompare90" )
   AAdd( aFuncList, "AdsMemICompare" )
   AAdd( aFuncList, "AdsMemICompare90" )
   AAdd( aFuncList, "AdsMemLwr" )
   AAdd( aFuncList, "AdsMemLwr90" )
   AAdd( aFuncList, "AdsMergeAOF" )
   AAdd( aFuncList, "AdsMgConnect" )
   AAdd( aFuncList, "AdsMgDisconnect" )
   AAdd( aFuncList, "AdsMgDumpInternalTables" )
   AAdd( aFuncList, "AdsMgGetActivityInfo" )
   AAdd( aFuncList, "AdsMgGetCommStats" )
   AAdd( aFuncList, "AdsMgGetConfigInfo" )
   AAdd( aFuncList, "AdsMgGetInstallInfo" )
   AAdd( aFuncList, "AdsMgGetLockOwner" )
   AAdd( aFuncList, "AdsMgGetLocks" )
   AAdd( aFuncList, "AdsMgGetOpenIndexes" )
   AAdd( aFuncList, "AdsMgGetOpenTables" )
   AAdd( aFuncList, "AdsMgGetServerType" )
   AAdd( aFuncList, "AdsMgGetUserNames" )
   AAdd( aFuncList, "AdsMgGetWorkerThreadActivity" )
   AAdd( aFuncList, "AdsMgKillUser" )
   AAdd( aFuncList, "AdsMgKillUser90" )
   AAdd( aFuncList, "AdsMgResetCommStats" )
   AAdd( aFuncList, "AdsNullTerminateStrings" )
   AAdd( aFuncList, "AdsOpenFileRaw" )
   AAdd( aFuncList, "AdsOpenIndex" )
   AAdd( aFuncList, "AdsOpenTable" )
   AAdd( aFuncList, "AdsOpenTable90" )
   AAdd( aFuncList, "AdsPackTable" )
   AAdd( aFuncList, "AdsParseExpr" )
   AAdd( aFuncList, "AdsParseExpr100" )
   AAdd( aFuncList, "AdsPerformRI" )
   AAdd( aFuncList, "AdsPrepareSQL" )
   AAdd( aFuncList, "AdsPrepareSQLNow" )
   AAdd( aFuncList, "AdsPrepareSQLNowW" )
   AAdd( aFuncList, "AdsPrepareSQLW" )
   AAdd( aFuncList, "AdsPullTrigger" )
   AAdd( aFuncList, "AdsReadFile" )
   AAdd( aFuncList, "AdsReadRecordNumbers" )
   AAdd( aFuncList, "AdsReadRecords" )
   AAdd( aFuncList, "AdsRecallAllRecords" )
   AAdd( aFuncList, "AdsRecallRecord" )
   AAdd( aFuncList, "AdsRefreshAOF" )
   AAdd( aFuncList, "AdsRefreshRecord" )
   AAdd( aFuncList, "AdsRefreshView" )
   AAdd( aFuncList, "AdsRegisterCallbackFunction" )
   AAdd( aFuncList, "AdsRegisterProgressCallback" )
   AAdd( aFuncList, "AdsRegisterSQLAbortFunc" )
   AAdd( aFuncList, "AdsRegisterUDF" )
   AAdd( aFuncList, "AdsReindex" )
   AAdd( aFuncList, "AdsReindex61" )
   AAdd( aFuncList, "AdsReindexFTS" )
   AAdd( aFuncList, "AdsReleaseObject" )
   AAdd( aFuncList, "AdsRemoveSQLComments" )
   AAdd( aFuncList, "AdsResetConnection" )
   AAdd( aFuncList, "AdsRestructureTable" )
   AAdd( aFuncList, "AdsRestructureTable90" )
   AAdd( aFuncList, "AdsRollbackTransaction" )
   AAdd( aFuncList, "AdsRollbackTransaction80" )
   AAdd( aFuncList, "AdsSeek" )
   AAdd( aFuncList, "AdsSeekInFile" )
   AAdd( aFuncList, "AdsSeekLast" )
   AAdd( aFuncList, "AdsSetAOF" )
   AAdd( aFuncList, "AdsSetAOF100" )
   AAdd( aFuncList, "AdsSetBOFFlag" )
   AAdd( aFuncList, "AdsSetBaseTableAccess" )
   AAdd( aFuncList, "AdsSetBinary" )
   AAdd( aFuncList, "AdsSetCollation" )
   AAdd( aFuncList, "AdsSetCollationLang" )
   AAdd( aFuncList, "AdsSetCollationSequence" )
   AAdd( aFuncList, "AdsSetCursorAOF" )
   AAdd( aFuncList, "AdsSetDate" )
   AAdd( aFuncList, "AdsSetDateFormat" )
   AAdd( aFuncList, "AdsSetDateFormat60" )
   AAdd( aFuncList, "AdsSetDecimals" )
   AAdd( aFuncList, "AdsSetDefault" )
   AAdd( aFuncList, "AdsSetDouble" )
   AAdd( aFuncList, "AdsSetEmpty" )
   AAdd( aFuncList, "AdsSetEpoch" )
   AAdd( aFuncList, "AdsSetExact" )
   AAdd( aFuncList, "AdsSetExact22" )
   AAdd( aFuncList, "AdsSetField" )
   AAdd( aFuncList, "AdsSetFieldRaw" )
   AAdd( aFuncList, "AdsSetFieldW" )
   AAdd( aFuncList, "AdsSetFilter" )
   AAdd( aFuncList, "AdsSetFilter100" )
   AAdd( aFuncList, "AdsSetFlushFlag" )
   AAdd( aFuncList, "AdsSetHandleLong" )
   AAdd( aFuncList, "AdsSetIndexDirection" )
   AAdd( aFuncList, "AdsSetInternalError" )
   AAdd( aFuncList, "AdsSetJulian" )
   AAdd( aFuncList, "AdsSetLastError" )
   AAdd( aFuncList, "AdsSetLogical" )
   AAdd( aFuncList, "AdsSetLong" )
   AAdd( aFuncList, "AdsSetLongLong" )
   AAdd( aFuncList, "AdsSetMilliseconds" )
   AAdd( aFuncList, "AdsSetMoney" )
   AAdd( aFuncList, "AdsSetNull" )
   AAdd( aFuncList, "AdsSetPacketSize" )
   AAdd( aFuncList, "AdsSetProperty" )
   AAdd( aFuncList, "AdsSetProperty90" )
   AAdd( aFuncList, "AdsSetRecord" )
   AAdd( aFuncList, "AdsSetRelKeyPos" )
   AAdd( aFuncList, "AdsSetRelation" )
   AAdd( aFuncList, "AdsSetRightsChecking" )
   AAdd( aFuncList, "AdsSetSQLTimeout" )
   AAdd( aFuncList, "AdsSetScope" )
   AAdd( aFuncList, "AdsSetScopedRelation" )
   AAdd( aFuncList, "AdsSetSearchPath" )
   AAdd( aFuncList, "AdsSetServerType" )
   AAdd( aFuncList, "AdsSetShort" )
   AAdd( aFuncList, "AdsSetString" )
   AAdd( aFuncList, "AdsSetStringW" )
   AAdd( aFuncList, "AdsSetTableCharType" )
   AAdd( aFuncList, "AdsSetTableTransactionFree" )
   AAdd( aFuncList, "AdsSetTime" )
   AAdd( aFuncList, "AdsSetTimeStamp" )
   AAdd( aFuncList, "AdsSetTimeStampRaw" )
   AAdd( aFuncList, "AdsSetupRI" )
   AAdd( aFuncList, "AdsShowDeleted" )
   AAdd( aFuncList, "AdsShowError" )
   AAdd( aFuncList, "AdsSkip" )
   AAdd( aFuncList, "AdsSkipUnique" )
   AAdd( aFuncList, "AdsSqlPeekStatement" )
   AAdd( aFuncList, "AdsStepIndexKey" )
   AAdd( aFuncList, "AdsStmtClearTablePasswords" )
   AAdd( aFuncList, "AdsStmtConstrainUpdates" )
   AAdd( aFuncList, "AdsStmtDisableEncryption" )
   AAdd( aFuncList, "AdsStmtEnableEncryption" )
   AAdd( aFuncList, "AdsStmtReadAllColumns" )
   AAdd( aFuncList, "AdsStmtSetOptimization" )
   AAdd( aFuncList, "AdsStmtSetTableCharType" )
   AAdd( aFuncList, "AdsStmtSetTableCollation" )
   AAdd( aFuncList, "AdsStmtSetTableLockType" )
   AAdd( aFuncList, "AdsStmtSetTablePassword" )
   AAdd( aFuncList, "AdsStmtSetTableReadOnly" )
   AAdd( aFuncList, "AdsStmtSetTableRights" )
   AAdd( aFuncList, "AdsStmtSetTableType" )
   AAdd( aFuncList, "AdsThreadExit" )
   AAdd( aFuncList, "AdsUnlockRecord" )
   AAdd( aFuncList, "AdsUnlockTable" )
   AAdd( aFuncList, "AdsVerifyPassword" )
   AAdd( aFuncList, "AdsVerifyRI" )
   AAdd( aFuncList, "AdsVerifySQL" )
   AAdd( aFuncList, "AdsVerifySQLW" )
   AAdd( aFuncList, "AdsWaitForObject" )
   AAdd( aFuncList, "AdsWriteAllRecords" )
   AAdd( aFuncList, "AdsWriteRecord" )
   AAdd( aFuncList, "AdsZapTable" )
   AAdd( aFuncList, "ObsAdsDecryptBuffer" )
   AAdd( aFuncList, "ObsAdsEncryptBuffer" )
   AAdd( aFuncList, "_AdsSetRecordPartial@12" )
   AAdd( aFuncList, "_WSFindAdvantage@12"     )
   AAdd( aFuncList, "_axAISRequest@16"        )
   AAdd( aFuncList, "_axCommReq@12"           )
   AAdd( aFuncList, "_axConnectLocal60@28"    )
   AAdd( aFuncList, "_axDisconnect@4"         )
   AAdd( aFuncList, "_axSetGaugeCallback@8"   )
   AAdd( aFuncList, "_axSetPacketSize@8"      )

   Return aFuncList

STATIC FUNCTION ACE32LIB_FUNCLIST()

   // These are interface functions exist in ace32.c
   // Based on ace32.dll version 10

   LOCAL aFuncList := {}

   AAdd( aFuncList, "AdsAddCustomKey" )
   AAdd( aFuncList, "AdsApplicationExit" )
   AAdd( aFuncList, "AdsAtEOF" )
   AAdd( aFuncList, "AdsBinaryToFile" )
   AAdd( aFuncList, "AdsBuildRawKey100" )
   AAdd( aFuncList, "AdsCacheOpenTables" )
   AAdd( aFuncList, "AdsCancelUpdate" )
   AAdd( aFuncList, "AdsCheckExistence" )
   AAdd( aFuncList, "AdsClearAllScopes" )
   AAdd( aFuncList, "AdsClearDefault" )
   AAdd( aFuncList, "AdsClearProgressCallback" )
   AAdd( aFuncList, "AdsClearSQLAbortFunc" )
   AAdd( aFuncList, "AdsClearScope" )
   AAdd( aFuncList, "AdsCloseAllIndexes" )
   AAdd( aFuncList, "AdsCloseCachedTables" )
   AAdd( aFuncList, "AdsCloseSQLStatement" )
   AAdd( aFuncList, "AdsCommitTransaction" )
   AAdd( aFuncList, "AdsConnect" )
   AAdd( aFuncList, "AdsConnect60" )
   AAdd( aFuncList, "AdsConvertTable" )
   AAdd( aFuncList, "AdsCopyTableContents" )
   AAdd( aFuncList, "AdsCreateFTSIndex" )
   AAdd( aFuncList, "AdsCreateIndex61" )
   AAdd( aFuncList, "AdsCreateSQLStatement" )
   AAdd( aFuncList, "AdsCreateTable" )
   AAdd( aFuncList, "AdsCreateTable90" )
   AAdd( aFuncList, "AdsDDAddIndexFile" )
   AAdd( aFuncList, "AdsDDAddProcedure100" )
   AAdd( aFuncList, "AdsDDAddTable90" )
   AAdd( aFuncList, "AdsDDAddView" )
   AAdd( aFuncList, "AdsDDCreate" )
   AAdd( aFuncList, "AdsDDCreateArticle100" )
   AAdd( aFuncList, "AdsDDCreatePublication" )
   AAdd( aFuncList, "AdsDDCreateRefIntegrity62" )
   AAdd( aFuncList, "AdsDDCreateTrigger" )
   AAdd( aFuncList, "AdsDDCreateUser" )
   AAdd( aFuncList, "AdsDDDeleteArticle" )
   AAdd( aFuncList, "AdsDDDeletePublication" )
   AAdd( aFuncList, "AdsDDDeleteUser" )
   AAdd( aFuncList, "AdsDDDeployDatabase" )
   AAdd( aFuncList, "AdsDDFindClose" )
   AAdd( aFuncList, "AdsDDFindNextObject" )
   AAdd( aFuncList, "AdsDDGetArticleProperty" )
   AAdd( aFuncList, "AdsDDGetFieldProperty" )
   AAdd( aFuncList, "AdsDDGetIndexProperty" )
   AAdd( aFuncList, "AdsDDGetPermissions" )
   AAdd( aFuncList, "AdsDDGetPublicationProperty" )
   AAdd( aFuncList, "AdsDDGetSubscriptionProperty" )
   AAdd( aFuncList, "AdsDDGetTriggerProperty" )
   AAdd( aFuncList, "AdsDDGetUserProperty" )
   AAdd( aFuncList, "AdsDDGrantPermission" )
   AAdd( aFuncList, "AdsDDMoveObjectFile" )
   AAdd( aFuncList, "AdsDDRemoveProcedure" )
   AAdd( aFuncList, "AdsDDRemoveTable" )
   AAdd( aFuncList, "AdsDDRemoveUserFromGroup" )
   AAdd( aFuncList, "AdsDDRenameObject" )
   AAdd( aFuncList, "AdsDDSetArticleProperty" )
   AAdd( aFuncList, "AdsDDSetFieldProperty" )
   AAdd( aFuncList, "AdsDDSetObjectAccessRights" )
   AAdd( aFuncList, "AdsDDSetPublicationProperty" )
   AAdd( aFuncList, "AdsDDSetTableProperty" )
   AAdd( aFuncList, "AdsDDSetUserProperty" )
   AAdd( aFuncList, "AdsDecryptRecord" )
   AAdd( aFuncList, "AdsDeleteCustomKey" )
   AAdd( aFuncList, "AdsDeleteIndex" )
   AAdd( aFuncList, "AdsDisableAutoIncEnforcement" )
   AAdd( aFuncList, "AdsDisableLocalConnections" )
   AAdd( aFuncList, "AdsDisableUniqueEnforcement" )
   AAdd( aFuncList, "AdsEnableAutoIncEnforcement" )
   AAdd( aFuncList, "AdsEnableRI" )
   AAdd( aFuncList, "AdsEncryptRecord" )
   AAdd( aFuncList, "AdsEvalAOF" )
   AAdd( aFuncList, "AdsEvalLogicalExpr" )
   AAdd( aFuncList, "AdsEvalNumericExpr" )
   AAdd( aFuncList, "AdsEvalTestExpr" )
   AAdd( aFuncList, "AdsExecuteSQLDirect" )
   AAdd( aFuncList, "AdsExtractKey" )
   AAdd( aFuncList, "AdsFileToBinary" )
   AAdd( aFuncList, "AdsFindConnection" )
   AAdd( aFuncList, "AdsFindFirstTable" )
   AAdd( aFuncList, "AdsFindNextTable" )
   AAdd( aFuncList, "AdsFindServers" )
   AAdd( aFuncList, "AdsGetAOF" )
   AAdd( aFuncList, "AdsGetAOFOptLevel" )
   AAdd( aFuncList, "AdsGetActiveLinkInfo" )
   AAdd( aFuncList, "AdsGetAllLocks" )
   AAdd( aFuncList, "AdsGetBinary" )
   AAdd( aFuncList, "AdsGetBookmark" )
   AAdd( aFuncList, "AdsGetBookmarkLength" )
   AAdd( aFuncList, "AdsGetCollationLang" )
   AAdd( aFuncList, "AdsGetConnectionProperty" )
   AAdd( aFuncList, "AdsGetDataLength" )
   AAdd( aFuncList, "AdsGetDateFormat" )
   AAdd( aFuncList, "AdsGetDecimals" )
   AAdd( aFuncList, "AdsGetDeleted" )
   AAdd( aFuncList, "AdsGetEpoch" )
   AAdd( aFuncList, "AdsGetExact" )
   AAdd( aFuncList, "AdsGetFTSIndexInfo" )
   AAdd( aFuncList, "AdsGetField" )
   AAdd( aFuncList, "AdsGetFieldLength" )
   AAdd( aFuncList, "AdsGetFieldName" )
   AAdd( aFuncList, "AdsGetFieldOffset" )
   AAdd( aFuncList, "AdsGetFieldType" )
   AAdd( aFuncList, "AdsGetFilter" )
   AAdd( aFuncList, "AdsGetHandleType" )
   AAdd( aFuncList, "AdsGetIndexCondition" )
   AAdd( aFuncList, "AdsGetIndexFilename" )
   AAdd( aFuncList, "AdsGetIndexHandleByExpr" )
   AAdd( aFuncList, "AdsGetIndexName" )
   AAdd( aFuncList, "AdsGetIntProperty" )
   AAdd( aFuncList, "AdsGetKeyColumn" )
   AAdd( aFuncList, "AdsGetKeyLength" )
   AAdd( aFuncList, "AdsGetKeyType" )
   AAdd( aFuncList, "AdsGetLastError" )
   AAdd( aFuncList, "AdsGetLogical" )
   AAdd( aFuncList, "AdsGetLongLong" )
   AAdd( aFuncList, "AdsGetMemoDataType" )
   AAdd( aFuncList, "AdsGetMilliseconds" )
   AAdd( aFuncList, "AdsGetNumActiveLinks" )
   AAdd( aFuncList, "AdsGetNumFields" )
   AAdd( aFuncList, "AdsGetNumLocks" )
   AAdd( aFuncList, "AdsGetNumParams" )
   AAdd( aFuncList, "AdsGetRecordCRC" )
   AAdd( aFuncList, "AdsGetRecordLength" )
   AAdd( aFuncList, "AdsGetRelKeyPos" )
   AAdd( aFuncList, "AdsGetSQLStatementHandle" )
   AAdd( aFuncList, "AdsGetSearchPath" )
   AAdd( aFuncList, "AdsGetServerTime" )
   AAdd( aFuncList, "AdsGetString" )
   AAdd( aFuncList, "AdsGetTableAlias" )
   AAdd( aFuncList, "AdsGetTableCollation" )
   AAdd( aFuncList, "AdsGetTableFilename" )
   AAdd( aFuncList, "AdsGetTableHandle25" )
   AAdd( aFuncList, "AdsGetTableMemoSize" )
   AAdd( aFuncList, "AdsGetTableRights" )
   AAdd( aFuncList, "AdsGetTime" )
   AAdd( aFuncList, "AdsGetVersion" )
   AAdd( aFuncList, "AdsGotoBookmark60" )
   AAdd( aFuncList, "AdsGotoRecord" )
   AAdd( aFuncList, "AdsImageToClipboard" )
   AAdd( aFuncList, "AdsInitRawKey" )
   AAdd( aFuncList, "AdsIsEmpty" )
   AAdd( aFuncList, "AdsIsExprValid" )
   AAdd( aFuncList, "AdsIsFound" )
   AAdd( aFuncList, "AdsIsIndexCompound" )
   AAdd( aFuncList, "AdsIsIndexDescending" )
   AAdd( aFuncList, "AdsIsIndexNullable" )
   AAdd( aFuncList, "AdsIsIndexUnique" )
   AAdd( aFuncList, "AdsIsNull" )
   AAdd( aFuncList, "AdsIsRecordDeleted" )
   AAdd( aFuncList, "AdsIsRecordInAOF" )
   AAdd( aFuncList, "AdsIsRecordVisible" )
   AAdd( aFuncList, "AdsIsTableEncrypted" )
   AAdd( aFuncList, "AdsIsTableTransactionFree" )
   AAdd( aFuncList, "AdsLockRecord" )
   AAdd( aFuncList, "AdsLookupKey" )
   AAdd( aFuncList, "AdsMgDisconnect" )
   AAdd( aFuncList, "AdsMgGetActivityInfo" )
   AAdd( aFuncList, "AdsMgGetConfigInfo" )
   AAdd( aFuncList, "AdsMgGetLockOwner" )
   AAdd( aFuncList, "AdsMgGetOpenIndexes" )
   AAdd( aFuncList, "AdsMgGetServerType" )
   AAdd( aFuncList, "AdsMgGetWorkerThreadActivity" )
   AAdd( aFuncList, "AdsMgKillUser90" )
   AAdd( aFuncList, "AdsNullTerminateStrings" )
   AAdd( aFuncList, "AdsOpenTable" )
   AAdd( aFuncList, "AdsPackTable" )
   AAdd( aFuncList, "AdsPrepareSQLW" )
   AAdd( aFuncList, "AdsRecallRecord" )
   AAdd( aFuncList, "AdsRefreshRecord" )
   AAdd( aFuncList, "AdsRegisterProgressCallback" )
   AAdd( aFuncList, "AdsRegisterUDF" )
   AAdd( aFuncList, "AdsReindex61" )
   AAdd( aFuncList, "AdsResetConnection" )
   AAdd( aFuncList, "AdsRestructureTable90" )
   AAdd( aFuncList, "AdsRollbackTransaction80" )
   AAdd( aFuncList, "AdsSeekLast" )
   AAdd( aFuncList, "AdsSetAOF100" )
   AAdd( aFuncList, "AdsSetCollation" )
   AAdd( aFuncList, "AdsSetDate" )
   AAdd( aFuncList, "AdsSetDateFormat60" )
   AAdd( aFuncList, "AdsSetDefault" )
   AAdd( aFuncList, "AdsSetEmpty" )
   AAdd( aFuncList, "AdsSetExact" )
   AAdd( aFuncList, "AdsSetField" )
   AAdd( aFuncList, "AdsSetFieldW" )
   AAdd( aFuncList, "AdsSetFilter100" )
   AAdd( aFuncList, "AdsSetIndexDirection" )
   AAdd( aFuncList, "AdsSetLogical" )
   AAdd( aFuncList, "AdsSetLongLong" )
   AAdd( aFuncList, "AdsSetMoney" )
   AAdd( aFuncList, "AdsSetRecord" )
   AAdd( aFuncList, "AdsSetRelation" )
   AAdd( aFuncList, "AdsSetSQLTimeout" )
   AAdd( aFuncList, "AdsSetScopedRelation" )
   AAdd( aFuncList, "AdsSetServerType" )
   AAdd( aFuncList, "AdsSetString" )
   AAdd( aFuncList, "AdsSetTableTransactionFree" )
   AAdd( aFuncList, "AdsSetTimeStamp" )
   AAdd( aFuncList, "AdsShowError" )
   AAdd( aFuncList, "AdsSkipUnique" )
   AAdd( aFuncList, "AdsStmtConstrainUpdates" )
   AAdd( aFuncList, "AdsStmtEnableEncryption" )
   AAdd( aFuncList, "AdsStmtSetTableCharType" )
   AAdd( aFuncList, "AdsStmtSetTableLockType" )
   AAdd( aFuncList, "AdsStmtSetTableReadOnly" )
   AAdd( aFuncList, "AdsStmtSetTableType" )
   AAdd( aFuncList, "AdsUnlockRecord" )
   AAdd( aFuncList, "AdsVerifyPassword" )
   AAdd( aFuncList, "AdsVerifySQLW" )
   AAdd( aFuncList, "AdsWriteRecord" )
   AAdd( aFuncList, "AdsAppendRecord" )
   AAdd( aFuncList, "AdsAtBOF" )
   AAdd( aFuncList, "AdsBeginTransaction" )
   AAdd( aFuncList, "AdsBuildRawKey" )
   AAdd( aFuncList, "AdsCacheOpenCursors" )
   AAdd( aFuncList, "AdsCacheRecords" )
   AAdd( aFuncList, "AdsCancelUpdate90" )
   AAdd( aFuncList, "AdsClearAOF" )
   AAdd( aFuncList, "AdsClearCallbackFunction" )
   AAdd( aFuncList, "AdsClearFilter" )
   AAdd( aFuncList, "AdsClearRelation" )
   AAdd( aFuncList, "AdsClearSQLParams" )
   AAdd( aFuncList, "AdsCloneTable" )
   AAdd( aFuncList, "AdsCloseAllTables" )
   AAdd( aFuncList, "AdsCloseIndex" )
   AAdd( aFuncList, "AdsCloseTable" )
   AAdd( aFuncList, "AdsCompareBookmarks" )
   AAdd( aFuncList, "AdsConnect26" )
   AAdd( aFuncList, "AdsContinue" )
   AAdd( aFuncList, "AdsCopyTable" )
   AAdd( aFuncList, "AdsCopyTableStructure" )
   AAdd( aFuncList, "AdsCreateIndex" )
   AAdd( aFuncList, "AdsCreateIndex90" )
   AAdd( aFuncList, "AdsCreateSavepoint" )
   AAdd( aFuncList, "AdsCreateTable71" )
   AAdd( aFuncList, "AdsCustomizeAOF" )
   AAdd( aFuncList, "AdsDDAddProcedure" )
   AAdd( aFuncList, "AdsDDAddTable" )
   AAdd( aFuncList, "AdsDDAddUserToGroup" )
   AAdd( aFuncList, "AdsDDAddView100" )
   AAdd( aFuncList, "AdsDDCreateArticle" )
   AAdd( aFuncList, "AdsDDCreateLink" )
   AAdd( aFuncList, "AdsDDCreateRefIntegrity" )
   AAdd( aFuncList, "AdsDDCreateSubscription" )
   AAdd( aFuncList, "AdsDDCreateTrigger100" )
   AAdd( aFuncList, "AdsDDCreateUserGroup" )
   AAdd( aFuncList, "AdsDDDeleteIndex" )
   AAdd( aFuncList, "AdsDDDeleteSubscription" )
   AAdd( aFuncList, "AdsDDDeleteUserGroup" )
   AAdd( aFuncList, "AdsDDDropLink" )
   AAdd( aFuncList, "AdsDDFindFirstObject" )
   AAdd( aFuncList, "AdsDDFreeTable" )
   AAdd( aFuncList, "AdsDDGetDatabaseProperty" )
   AAdd( aFuncList, "AdsDDGetIndexFileProperty" )
   AAdd( aFuncList, "AdsDDGetLinkProperty" )
   AAdd( aFuncList, "AdsDDGetProcedureProperty" )
   AAdd( aFuncList, "AdsDDGetRefIntegrityProperty" )
   AAdd( aFuncList, "AdsDDGetTableProperty" )
   AAdd( aFuncList, "AdsDDGetUserGroupProperty" )
   AAdd( aFuncList, "AdsDDGetViewProperty" )
   AAdd( aFuncList, "AdsDDModifyLink" )
   AAdd( aFuncList, "AdsDDRemoveIndexFile" )
   AAdd( aFuncList, "AdsDDRemoveRefIntegrity" )
   AAdd( aFuncList, "AdsDDRemoveTrigger" )
   AAdd( aFuncList, "AdsDDRemoveView" )
   AAdd( aFuncList, "AdsDDRevokePermission" )
   AAdd( aFuncList, "AdsDDSetDatabaseProperty" )
   AAdd( aFuncList, "AdsDDSetIndexProperty" )
   AAdd( aFuncList, "AdsDDSetProcedureProperty" )
   AAdd( aFuncList, "AdsDDSetSubscriptionProperty" )
   AAdd( aFuncList, "AdsDDSetUserGroupProperty" )
   AAdd( aFuncList, "AdsDDSetViewProperty" )
   AAdd( aFuncList, "AdsDecryptTable" )
   AAdd( aFuncList, "AdsDeleteFile" )
   AAdd( aFuncList, "AdsDeleteRecord" )
   AAdd( aFuncList, "AdsDisableEncryption" )
   AAdd( aFuncList, "AdsDisableRI" )
   AAdd( aFuncList, "AdsDisconnect" )
   AAdd( aFuncList, "AdsEnableEncryption" )
   AAdd( aFuncList, "AdsEnableUniqueEnforcement" )
   AAdd( aFuncList, "AdsEncryptTable" )
   AAdd( aFuncList, "AdsEvalAOF100" )
   AAdd( aFuncList, "AdsEvalLogicalExprW" )
   AAdd( aFuncList, "AdsEvalStringExpr" )
   AAdd( aFuncList, "AdsExecuteSQL" )
   AAdd( aFuncList, "AdsExecuteSQLDirectW" )
   AAdd( aFuncList, "AdsFailedTransactionRecovery" )
   AAdd( aFuncList, "AdsFindClose" )
   AAdd( aFuncList, "AdsFindConnection25" )
   AAdd( aFuncList, "AdsFindFirstTable62" )
   AAdd( aFuncList, "AdsFindNextTable62" )
   AAdd( aFuncList, "AdsFlushFileBuffers" )
   AAdd( aFuncList, "AdsGetAOF100" )
   AAdd( aFuncList, "AdsGetAOFOptLevel100" )
   AAdd( aFuncList, "AdsGetAllIndexes" )
   AAdd( aFuncList, "AdsGetAllTables" )
   AAdd( aFuncList, "AdsGetBinaryLength" )
   AAdd( aFuncList, "AdsGetBookmark60" )
   AAdd( aFuncList, "AdsGetCollation" )
   AAdd( aFuncList, "AdsGetConnectionPath" )
   AAdd( aFuncList, "AdsGetConnectionType" )
   AAdd( aFuncList, "AdsGetDate" )
   AAdd( aFuncList, "AdsGetDateFormat60" )
   AAdd( aFuncList, "AdsGetDefault" )
   AAdd( aFuncList, "AdsGetDouble" )
   AAdd( aFuncList, "AdsGetErrorString" )
   AAdd( aFuncList, "AdsGetExact22" )
   AAdd( aFuncList, "AdsGetFTSIndexes" )
   AAdd( aFuncList, "AdsGetFieldDecimals" )
   AAdd( aFuncList, "AdsGetFieldLength100" )
   AAdd( aFuncList, "AdsGetFieldNum" )
   AAdd( aFuncList, "AdsGetFieldRaw" )
   AAdd( aFuncList, "AdsGetFieldW" )
   AAdd( aFuncList, "AdsGetHandleLong" )
   AAdd( aFuncList, "AdsGetIndexCollation" )
   AAdd( aFuncList, "AdsGetIndexExpr" )
   AAdd( aFuncList, "AdsGetIndexHandle" )
   AAdd( aFuncList, "AdsGetIndexHandleByOrder" )
   AAdd( aFuncList, "AdsGetIndexOrderByHandle" )
   AAdd( aFuncList, "AdsGetJulian" )
   AAdd( aFuncList, "AdsGetKeyCount" )
   AAdd( aFuncList, "AdsGetKeyNum" )
   AAdd( aFuncList, "AdsGetLastAutoinc" )
   AAdd( aFuncList, "AdsGetLastTableUpdate" )
   AAdd( aFuncList, "AdsGetLong" )
   AAdd( aFuncList, "AdsGetMemoBlockSize" )
   AAdd( aFuncList, "AdsGetMemoLength" )
   AAdd( aFuncList, "AdsGetMoney" )
   AAdd( aFuncList, "AdsGetNumFTSIndexes" )
   AAdd( aFuncList, "AdsGetNumIndexes" )
   AAdd( aFuncList, "AdsGetNumOpenTables" )
   AAdd( aFuncList, "AdsGetRecord" )
   AAdd( aFuncList, "AdsGetRecordCount" )
   AAdd( aFuncList, "AdsGetRecordNum" )
   AAdd( aFuncList, "AdsGetSQLStatement" )
   AAdd( aFuncList, "AdsGetScope" )
   AAdd( aFuncList, "AdsGetServerName" )
   AAdd( aFuncList, "AdsGetShort" )
   AAdd( aFuncList, "AdsGetStringW" )
   AAdd( aFuncList, "AdsGetTableCharType" )
   AAdd( aFuncList, "AdsGetTableConnection" )
   AAdd( aFuncList, "AdsGetTableHandle" )
   AAdd( aFuncList, "AdsGetTableLockType" )
   AAdd( aFuncList, "AdsGetTableOpenOptions" )
   AAdd( aFuncList, "AdsGetTableType" )
   AAdd( aFuncList, "AdsGetTransactionCount" )
   AAdd( aFuncList, "AdsGotoBookmark" )
   AAdd( aFuncList, "AdsGotoBottom" )
   AAdd( aFuncList, "AdsGotoTop" )
   AAdd( aFuncList, "AdsInTransaction" )
   AAdd( aFuncList, "AdsIsConnectionAlive" )
   AAdd( aFuncList, "AdsIsEncryptionEnabled" )
   AAdd( aFuncList, "AdsIsFieldBinary" )
   AAdd( aFuncList, "AdsIsIndexCandidate" )
   AAdd( aFuncList, "AdsIsIndexCustom" )
   AAdd( aFuncList, "AdsIsIndexFTS" )
   AAdd( aFuncList, "AdsIsIndexPrimaryKey" )
   AAdd( aFuncList, "AdsIsIndexUserDefined" )
   AAdd( aFuncList, "AdsIsNullable" )
   AAdd( aFuncList, "AdsIsRecordEncrypted" )
   AAdd( aFuncList, "AdsIsRecordLocked" )
   AAdd( aFuncList, "AdsIsServerLoaded" )
   AAdd( aFuncList, "AdsIsTableLocked" )
   AAdd( aFuncList, "AdsLocate" )
   AAdd( aFuncList, "AdsLockTable" )
   AAdd( aFuncList, "AdsMgConnect" )
   AAdd( aFuncList, "AdsMgDumpInternalTables" )
   AAdd( aFuncList, "AdsMgGetCommStats" )
   AAdd( aFuncList, "AdsMgGetInstallInfo" )
   AAdd( aFuncList, "AdsMgGetLocks" )
   AAdd( aFuncList, "AdsMgGetOpenTables" )
   AAdd( aFuncList, "AdsMgGetUserNames" )
   AAdd( aFuncList, "AdsMgKillUser" )
   AAdd( aFuncList, "AdsMgResetCommStats" )
   AAdd( aFuncList, "AdsOpenIndex" )
   AAdd( aFuncList, "AdsOpenTable90" )
   AAdd( aFuncList, "AdsPrepareSQL" )
   AAdd( aFuncList, "AdsRecallAllRecords" )
   AAdd( aFuncList, "AdsRefreshAOF" )
   AAdd( aFuncList, "AdsRegisterCallbackFunction" )
   AAdd( aFuncList, "AdsRegisterSQLAbortFunc" )
   AAdd( aFuncList, "AdsReindex" )
   AAdd( aFuncList, "AdsReindexFTS" )
   AAdd( aFuncList, "AdsRestructureTable" )
   AAdd( aFuncList, "AdsRollbackTransaction" )
   AAdd( aFuncList, "AdsSeek" )
   AAdd( aFuncList, "AdsSetAOF" )
   AAdd( aFuncList, "AdsSetBinary" )
   AAdd( aFuncList, "AdsSetCollationLang" )
   AAdd( aFuncList, "AdsSetDateFormat" )
   AAdd( aFuncList, "AdsSetDecimals" )
   AAdd( aFuncList, "AdsSetDouble" )
   AAdd( aFuncList, "AdsSetEpoch" )
   AAdd( aFuncList, "AdsSetExact22" )
   AAdd( aFuncList, "AdsSetFieldRaw" )
   AAdd( aFuncList, "AdsSetFilter" )
   AAdd( aFuncList, "AdsSetHandleLong" )
   AAdd( aFuncList, "AdsSetJulian" )
   AAdd( aFuncList, "AdsSetLong" )
   AAdd( aFuncList, "AdsSetMilliseconds" )
   AAdd( aFuncList, "AdsSetNull" )
   AAdd( aFuncList, "AdsSetRelKeyPos" )
   AAdd( aFuncList, "AdsSetRightsChecking" )
   AAdd( aFuncList, "AdsSetScope" )
   AAdd( aFuncList, "AdsSetSearchPath" )
   AAdd( aFuncList, "AdsSetShort" )
   AAdd( aFuncList, "AdsSetStringW" )
   AAdd( aFuncList, "AdsSetTime" )
   AAdd( aFuncList, "AdsShowDeleted" )
   AAdd( aFuncList, "AdsSkip" )
   AAdd( aFuncList, "AdsStmtClearTablePasswords" )
   AAdd( aFuncList, "AdsStmtDisableEncryption" )
   AAdd( aFuncList, "AdsStmtReadAllColumns" )
   AAdd( aFuncList, "AdsStmtSetTableCollation" )
   AAdd( aFuncList, "AdsStmtSetTablePassword" )
   AAdd( aFuncList, "AdsStmtSetTableRights" )
   AAdd( aFuncList, "AdsThreadExit" )
   AAdd( aFuncList, "AdsUnlockTable" )
   AAdd( aFuncList, "AdsVerifySQL" )
   AAdd( aFuncList, "AdsWriteAllRecords" )
   AAdd( aFuncList, "AdsZapTable" )

   RETURN aFuncList
